/*
  tapo.cpp - TP-Link TAPO Client for P100/P105/P110

  Copyright (c) 2020-2023 Sasapea's Lab. All right reserved.

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General
  Public License along with this library; if not, write to the
  Free Software Foundation, Inc., 59 Temple Place, Suite 330,
  Boston, MA  02111-1307  USA
*/
#include "tapo.h"

TAPO tapo;

void usage()
{
  fprintf(stderr, "\n");
  fprintf(stderr, "TP-Link Tapo Smart Plug Client v1.1\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "Usage: tapo [-h] ipaddress username password info|on|off|energy\n");
  fprintf(stderr, "\n");
  fprintf(stderr, "optional arguments:\n");
  fprintf(stderr, "  -h ... show this help\n");
}

int main(int argc, char *argv[])
{
  const char *hostname = "";
  const char *username = "";
  const char *password = "";
  const char *command  = "";
  bool  help = false;
  int exitCode = EXIT_SUCCESS;
  for (int i = 1; i < argc; ++i)
  {
    char *p = argv[i];
    if (*p == '-')
    {
      switch (p[1])
      {
        default:
          exitCode = EXIT_FAILURE;
        case 'h':
          help = true;
          break;
      }
    }
    else
    {
      switch (i)
      {
        case 1: hostname = p; break;
        case 2: username = p; break;
        case 3: password = p; break;
        case 4: command  = p; break;
        default:
          exitCode = EXIT_FAILURE;
          help = true;
          break;
      }
    }
  }
  if (!help)
  {
    if (!*hostname || !*username || !*password || !*command)
    {
      exitCode = EXIT_FAILURE;
      help = true;
    }
    else
    {
      bool rv = false;
      tapo.begin(hostname, username, password);
      if (strcmp("energy", command) == 0)
      {
        std::string info;
        rv = tapo.getEnergyUsage(info);
        if (rv)
          fprintf(stdout, "%s\n", info.c_str());
      }
      else if (strcmp("info", command) == 0)
      {
        std::string info;
        rv = tapo.getDeviceInfo(info);
        if (rv)
          fprintf(stdout, "%s\n", info.c_str());
      }
      else if (strcmp("on", command) == 0)
      {
        rv = tapo.setDeviceOn(true);
      }
      else if (strcmp("off", command) == 0)
      {
        rv = tapo.setDeviceOn(false);
      }
      else
      {
        fprintf(stderr, "invalid command [%s]\n", command);
        help = true;
      }
      tapo.end();
      if (!help && !rv)
      {
        exitCode = EXIT_FAILURE;
        if (tapo.httpStatus() != HTTP_CODE_OK)
          fprintf(stderr, "HTTP Status: [%d]\n", tapo.httpStatus());
        else if (tapo.tapoStatus())
          fprintf(stderr, "TAPO Status: [%d]\n", tapo.tapoStatus());
        else if (tapo.mbedStatus())
          fprintf(stderr, "MBEDTLS Status: -0x[%04X]\n", abs(tapo.mbedStatus()));
      }
    }
  }
  if (help)
    usage();
  return exitCode;
}
